@@ -32,8 +32,22 @@ header <- function(label,
 
 }
 
-startswith <- function(string, prefix) {
-  substring(string, 1, nchar(prefix)) == prefix
+heredoc <- function(text, leave = 0) {
+
+  # remove leading, trailing whitespace
+  trimmed <- gsub("^\\s*\\n|\\n\\s*$", "", text)
+
+  # split into lines
+  lines <- strsplit(trimmed, "\n", fixed = TRUE)[[1L]]
+
+  # compute common indent
+  indent <- regexpr("[^[:space:]]", lines)
+  common <- min(setdiff(indent, -1L)) - leave
+  text <- paste(substring(lines, common), collapse = "\n")
+
+  # substitute in ANSI links for executable renv code
+  ansify(text)
+
 }
 
 bootstrap <- function(version, library) {
@@ -191,8 +205,11 @@ renv_bootstrap_download_impl <- function(url, destfile) {
     quiet    = TRUE
   )
 
-  if ("headers" %in% names(formals(utils::download.file)))
-    args$headers <- renv_bootstrap_download_custom_headers(url)
+  if ("headers" %in% names(formals(utils::download.file))) {
+    headers <- renv_bootstrap_download_custom_headers(url)
+    if (length(headers) && is.character(headers))
+      args$headers <- headers
+  }
 
   do.call(utils::download.file, args)
 
@@ -271,10 +288,21 @@ renv_bootstrap_download_cran_latest_find <- function(version) {
   for (type in types) {
     for (repos in renv_bootstrap_repos()) {
 
+      # build arguments for utils::available.packages() call
+      args <- list(type = type, repos = repos)
+
+      # add custom headers if available -- note that
+      # utils::available.packages() will pass this to download.file()
+      if ("headers" %in% names(formals(utils::download.file))) {
+        headers <- renv_bootstrap_download_custom_headers(repos)
+        if (length(headers) && is.character(headers))
+          args$headers <- headers
+      }
+
       # retrieve package database
       db <- tryCatch(
         as.data.frame(
-          utils::available.packages(type = type, repos = repos),
+          do.call(utils::available.packages, args),
           stringsAsFactors = FALSE
         ),
         error = identity
@@ -356,6 +384,14 @@ renv_bootstrap_download_tarball <- function(version) {
 
 }
 
+renv_bootstrap_github_token <- function() {
+  for (envvar in c("GITHUB_TOKEN", "GITHUB_PAT", "GH_TOKEN")) {
+    envval <- Sys.getenv(envvar, unset = NA)
+    if (!is.na(envval))
+      return(envval)
+  }
+}
+
 renv_bootstrap_download_github <- function(version) {
 
   enabled <- Sys.getenv("RENV_BOOTSTRAP_FROM_GITHUB", unset = "TRUE")
@@ -363,16 +399,19 @@ renv_bootstrap_download_github <- function(version) {
     return(FALSE)
 
   # prepare download options
-  pat <- Sys.getenv("GITHUB_PAT")
-  if (nzchar(Sys.which("curl")) && nzchar(pat)) {
+  token <- renv_bootstrap_github_token()
+  if (is.null(token))
+    token <- ""
+
+  if (nzchar(Sys.which("curl")) && nzchar(token)) {
     fmt <- "--location --fail --header \"Authorization: token %s\""
-    extra <- sprintf(fmt, pat)
+    extra <- sprintf(fmt, token)
     saved <- options("download.file.method", "download.file.extra")
     options(download.file.method = "curl", download.file.extra = extra)
     on.exit(do.call(base::options, saved), add = TRUE)
-  } else if (nzchar(Sys.which("wget")) && nzchar(pat)) {
+  } else if (nzchar(Sys.which("wget")) && nzchar(token)) {
     fmt <- "--header=\"Authorization: token %s\""
-    extra <- sprintf(fmt, pat)
+    extra <- sprintf(fmt, token)
     saved <- options("download.file.method", "download.file.extra")
     options(download.file.method = "wget", download.file.extra = extra)
     on.exit(do.call(base::options, saved), add = TRUE)
@@ -443,7 +482,7 @@ renv_bootstrap_git_extract_sha1_tar <- function(bundle) {
 
   # open the bundle for reading
   # We use gzcon for everything because (from ?gzcon)
-  # > Reading from a connection which does not supply a ‘gzip’ magic
+  # > Reading from a connection which does not supply a 'gzip' magic
   # > header is equivalent to reading from the original connection
   conn <- gzcon(file(bundle, open = "rb", raw = TRUE))
   on.exit(close(conn))
@@ -534,6 +573,9 @@ renv_bootstrap_platform_prefix_impl <- function() {
 
   # if the user has requested an automatic prefix, generate it
   auto <- Sys.getenv("RENV_PATHS_PREFIX_AUTO", unset = NA)
+  if (is.na(auto) && getRversion() >= "4.4.0")
+    auto <- "TRUE"
+
   if (auto %in% c("TRUE", "True", "true", "1"))
     return(renv_bootstrap_platform_prefix_auto())
 
@@ -706,10 +748,12 @@ renv_bootstrap_library_root_impl <- function(project) {
 renv_bootstrap_validate_version <- function(version, description = NULL) {
 
   # resolve description file
-  description <- description %||% {
-    path <- getNamespaceInfo("renv", "path")
-    packageDescription("renv", lib.loc = dirname(path))
-  }
+  #
+  # avoid passing lib.loc to `packageDescription()` below, since R will
+  # use the loaded version of the package by default anyhow. note that
+  # this function should only be called after 'renv' is loaded
+  # https://github.com/rstudio/renv/issues/1625
+  description <- description %||% packageDescription("renv")
 
   # check whether requested version 'version' matches loaded version of renv
   sha <- attr(version, "sha", exact = TRUE)
@@ -723,24 +767,23 @@ renv_bootstrap_validate_version <- function(version, description = NULL) {
 
   # the loaded version of renv doesn't match the requested version;
   # give the user instructions on how to proceed
-  remote <- if (!is.null(description[["RemoteSha"]])) {
+  dev <- identical(description[["RemoteType"]], "github")
+  remote <- if (dev)
     paste("rstudio/renv", description[["RemoteSha"]], sep = "@")
-  } else {
+  else
     paste("renv", description[["Version"]], sep = "@")
-  }
 
   # display both loaded version + sha if available
   friendly <- renv_bootstrap_version_friendly(
     version = description[["Version"]],
-    sha     = description[["RemoteSha"]]
+    sha     = if (dev) description[["RemoteSha"]]
   )
 
-  fmt <- paste(
-    "renv %1$s was loaded from project library, but this project is configured to use renv %2$s.",
-    "- Use `renv::record(\"%3$s\")` to record renv %1$s in the lockfile.",
-    "- Use `renv::restore(packages = \"renv\")` to install renv %2$s into the project library.",
-    sep = "\n"
-  )
+  fmt <- heredoc("
+    renv %1$s was loaded from project library, but this project is configured to use renv %2$s.
+    - Use `renv::record(\"%3$s\")` to record renv %1$s in the lockfile.
+    - Use `renv::restore(packages = \"renv\")` to install renv %2$s into the project library.
+  ")
   catf(fmt, friendly, renv_bootstrap_version_friendly(version), remote)
 
   FALSE
@@ -748,8 +791,14 @@ renv_bootstrap_validate_version <- function(version, description = NULL) {
 }
 
 renv_bootstrap_validate_version_dev <- function(version, description) {
+  
   expected <- description[["RemoteSha"]]
-  is.character(expected) && startswith(expected, version)
+  if (!is.character(expected))
+    return(FALSE)
+  
+  pattern <- sprintf("^\\Q%s\\E", version)
+  grepl(pattern, expected, perl = TRUE)
+  
 }
 
 renv_bootstrap_validate_version_release <- function(version, description) {
@@ -780,7 +829,7 @@ renv_bootstrap_load <- function(project, libpath, version) {
   hooks <- getHook("renv::autoload")
   for (hook in hooks)
     if (is.function(hook))
-      tryCatch(hook(), error = warning)
+      tryCatch(hook(), error = warnify)
 
   # load the project
   renv::load(project)
@@ -921,13 +970,18 @@ renv_bootstrap_user_dir_impl <- function() {
 
 }
 
-renv_bootstrap_version_friendly <- function(version, sha = NULL) {
+renv_bootstrap_version_friendly <- function(version, shafmt = NULL, sha = NULL) {
   sha <- sha %||% attr(version, "sha", exact = TRUE)
-  parts <- c(version, sprintf("[sha: %s]", substring(sha, 1L, 7L)))
-  paste(parts, collapse = " ")
+  parts <- c(version, sprintf(shafmt %||% " [sha: %s]", substring(sha, 1L, 7L)))
+  paste(parts, collapse = "")
 }
 
-renv_bootstrap_run <- function(version, libpath) {
+renv_bootstrap_exec <- function(project, libpath, version) {
+  if (!renv_bootstrap_load(project, libpath, version))
+    renv_bootstrap_run(project, libpath, version)
+}
+
+renv_bootstrap_run <- function(project, libpath, version) {
 
   # perform bootstrap
   bootstrap(version, libpath)
@@ -938,7 +992,7 @@ renv_bootstrap_run <- function(version, libpath) {
 
   # try again to load
   if (requireNamespace("renv", lib.loc = libpath, quietly = TRUE)) {
-    return(renv::load(project = getwd()))
+    return(renv::load(project = project))
   }
 
   # failed to download or load renv; warn the user
@@ -950,8 +1004,3 @@ renv_bootstrap_run <- function(version, libpath) {
   warning(paste(msg, collapse = "\n"), call. = FALSE)
 
 }
-
-
-renv_bootstrap_in_rstudio <- function() {
-  commandArgs()[[1]] == "RStudio"
-}
